#include <stdio.h>
#include <string.h>
#include "addrem.h"

/* Removes a header from the file specified by 'fname.'
 * - Returns 0 on success, otherwise it returns an error code (see addrem.h)
 * - Switches can be combined with the Binary Or (|) operator. */
int remHeader(char *fname, int switches) {
	FILE *rom, *temp;
	int fsize;

	rom = fopen(fname, "rb");
	if (rom == NULL) /* Couldn't open file for reading */
		return ERROR_OPENREAD;
	else {
		char buffer[0x200]; /* The size of an SNES header */
		size_t num_read;

		fseek(rom, 0, SEEK_END);
		fsize = ftell(rom);

		/* Make sure the rom is legit */
		if (!(switches & SWITCH_FORCE)) {
			if (fsize < 0x7fff) /* It could be an honest mistake :P */
				return ERROR_TOOSMALL;
			else if ((fsize & 0x7fff) != 0x200) /* Tests for a header */
				return ERROR_NOHEADER;
		}

		/* Create a backup, if requested */

		if (switches & SWITCH_BACKUP) {
			FILE *backup_rom;
			char *backup_fname = malloc(strlen(fname) + strlen(BACKUP_SUFFIX));

			strcpy(backup_fname, fname); 
			strcat(backup_fname, BACKUP_SUFFIX);

			backup_rom = fopen(backup_fname, "wb");
			if (backup_rom == NULL) /* Backup didn't save */
				return ERROR_NOBACKUP;

			rewind(rom);
			int i;
			for (i=0;!feof(rom);++i) {
				num_read = fread(buffer, 1, 0x200, rom);
				fwrite(buffer, 1, num_read, backup_rom);
			}
			fclose(backup_rom);
		}

	fseek(rom, 0x200, SEEK_SET); /* Skip over the header */

		temp = fopen(TEMPNAME, "wb");
		int i;
		for (i=0;!feof(rom);++i)
		{
			num_read = fread(buffer, 1, 0x200, rom);
			fwrite(buffer, 1, num_read, temp);
		}
		fclose(rom);
		fclose(temp);

		remove(fname);
		rename(TEMPNAME, fname);

		return 0;
	}
}

/*************************/
/* Adds a header to the file specified by 'fname.'
 * - Returns 0 on success, otherwise it returns an error code (see addrem.h)
 * - Switches can be combined with the Binary Or (|) operator. */
int addHeader(char *fname, int switches) {
	FILE *rom, *temp;
	int fsize;

	rom = fopen(fname, "rb");
	if (rom == NULL) /* Couldn't open file for reading */
		return ERROR_OPENREAD;
	else {
		char buffer[0x200]; /* The size of an SNES header */
		size_t num_read;

		fseek(rom, 0, SEEK_END);
		fsize = ftell(rom);

		/* Make sure the rom is legit */
		if (!(switches & SWITCH_FORCE)) {
			if (fsize < 0x7fff) /* It could be an honest mistake :P */
				return ERROR_TOOSMALL;
			else if ((fsize & 0x7fff) == 0x200) /* Tests for a header */
				return ERROR_HEADER;
		}

		/* Create a backup, if requested */
		if (switches & SWITCH_BACKUP) {
			FILE *backup_rom;
			char *backup_fname = malloc(strlen(fname) + strlen(BACKUP_SUFFIX));

			strcpy(backup_fname, fname);
			strcat(backup_fname, BACKUP_SUFFIX);

			backup_rom = fopen(backup_fname, "wb");
			if (backup_rom == NULL) /* Backup didn't save */
				return ERROR_NOBACKUP;

			rewind(rom);
			int i;
			for (i=0;!feof(rom);++i) {
				num_read = fread(buffer, 1, 0x200, rom);
				fwrite(buffer, 1, num_read, backup_rom);
			}
			fclose(backup_rom);
		}
		rewind(rom);

		temp = fopen(TEMPNAME, "wb");
		memset(buffer, 0, 0x200); /* Fill buffer with zeros */
		fwrite(buffer, 1, 0x200, temp); /* Write empty header */

		int i;
		for (i=0;!feof(rom);++i)
		{
			num_read = fread(buffer, 1, 0x200, rom);
			fwrite(buffer, 1, num_read, temp);
		}
		fclose(rom);
		fclose(temp);

		remove(fname);
		rename(TEMPNAME, fname);

		return 0;
	}
}

