#include <stdio.h>
#include <string.h>
#include "addrem.h"
#include "stripsnes.h"

int main(int argc, char *argv[]) {
	int switches = 0;
	char *fname = "";

	if (getArgs(argc, argv, &fname, &switches)) {
		printUsage();
		return 1;
	}
	if (fname == (char *)"") {
		printUsage();
		return 1;
	}

	if (switches & SWITCH_ADD)
		return handleErrors(addHeader(fname, switches));
	else
		return handleErrors(remHeader(fname, switches));
}

/* Prints usage info. 'stripsnes' is the filename used to call the program */
void printUsage(void) {
	printf(
		"  stripsnes v%s\n"
		"    By Killa B\n"
		"  \n"
		"  Usage:\n"
		"  	 stripsnes [switches] [filename]\n"
		"  \n"
		"  Options:\n"
		"    -A, --add                 Adds a header, instead of removing one\n"
		"    -F, --force               Forces operation even if the file does not\n"
		"                                appear to have a header\n"
		"    -B, --backup              Make a backup of the rom. Should always be\n"
		"                                used when using -F\n", VERSION);
}

/* Gets commandline parameters and returns true if there's more than one. */
int getArgs(int argc, char *argv[], char* fname[], int* switches) {
	int i;
	for (i=1;i<argc;++i) {
		if (argv[i][0] == '-') { /* A switch */
			if (argv[i][1] == '-') { /* Long switch */
				if (!strcmp(argv[i], "--add"))
					*switches |= SWITCH_ADD;
				else if (!strcmp(argv[i], "--force"))
					*switches |= SWITCH_FORCE;
				else if (!strcmp(argv[i], "--backup"))
					*switches |= SWITCH_BACKUP;
				else {
					printf("Unrecognized option \"%s\"\n", argv[i]);
					return 1;
				}
			}
			else { /* Short switch */
				int j;
				for (j=1;j<strlen(argv[i]);++j) {
					if (argv[i][j] == 'A')
						*switches |= SWITCH_ADD;
					else if (argv[i][j] == 'F')
						*switches |= SWITCH_FORCE;
					else if (argv[i][j] == 'B')
						*switches |= SWITCH_BACKUP;
					else {
						printf("Unrecognized option \"-%c\"\n", argv[i][j]);
						return 1;
					}
				}
			}
		}
		else { /* Filename */
			if (*fname == (char *)"")
				*fname = argv[i];
			else { /* Uh-oh */
				printf("Unexpected input - \"%s\"\n", argv[i]);
				return 1;
			}
		}
	}
	return 0;
}

int handleErrors(int error) {
	switch (error) {
		case (ERROR_OPENREAD): {
			perror("Error");
			printf("Unable to open file for reading.");
		}
		break;
		case (ERROR_TOOSMALL):
			printf("There's no way that's an SNES rom, man.\n"
				"Quit screwing around or use \"--force\"\n");
		break;
		case (ERROR_NOHEADER):
			printf("This rom doesn't appear to have a header.\n"
				"Use \"--force\" if you're really sure it does.\n");
		break;
		case (ERROR_HEADER):
			printf("This rom appears to already have a header.\n"
				"Use \"--force\" if you're really sure it doesn't.\n");
		break;
		case (ERROR_NOBACKUP): {
			perror("Error");
			printf("Unable to create backup\n");
		}
		break;
	}
	return error;
}

